
/*
	Interface.c contains the top-level control window
	(with the main menu) and the primary display window.


/*		CRYSTAL IMAGE 0.9.0 BETA
		This program is copyrighted material.
		(C) 2002-2010 by Mark A. Haidekker, all rights reserved.
		
		This program is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE:

		THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
		APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
		HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
		OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
		THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
		PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
		IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
		ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

		IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
		WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
		THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
		GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
		USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
		DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
		PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
		EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
		SUCH DAMAGES.

		For the full text of the license, please see the file COPYING
		in the project's main directory, or read it on the Web at
		http://www.gnu.org/copyleft/gpl.html

 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>


#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "application.h"
#include "gtkhelpers.h"
#include "magnifier.h"


GtkWidget* create_window0 (void)
{
GtkWidget *window0;
	
	window0 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (window0), "window0", window0);
	gtk_window_set_title (GTK_WINDOW (window0), "Crystal Image Control");

	return window0;
}


GtkWidget* create_window1 (void)
{
  GtkWidget *vbox1, *vbox2, *vbox3, *vbox4, *hbox1, *table1;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;
  GtkWidget *button4;
  GtkWidget* label1;
  GtkWidget *vseparator1, *hsep1, *hsep2;
  GtkWidget *thumbnail;


	/* Menu */

  GtkWidget *menubar1;

	/* Menu: FILE */

  GtkWidget *file1;
  GtkWidget *file1_menu;
  GtkAccelGroup *file1_menu_accels;
  GtkWidget *open1;
  GtkWidget *open_multiple1;
  GtkWidget *import1;
  GtkWidget *rgbload1;
  GtkWidget *close1;
  GtkWidget *save1;
  GtkWidget *prefs1;
  GtkWidget *quit1;

	/* Menu: EDIT */

  GtkWidget *editm;
  GtkWidget *edit_menu;
  GtkAccelGroup *edit_menu_accels;
  GtkWidget *edit_undo;
  GtkWidget *edit_trim;
  GtkWidget *edit_crop;
  GtkWidget *edit_rotzoom;
  GtkWidget *edit_rebin;
  GtkWidget *edit_datatype;
  GtkWidget *edit_clickstk;
  GtkWidget *edit_values;
  
	/* Menu: VIEW */

  GtkWidget *view1;
  GtkWidget *view1_menu;
  GtkAccelGroup *view1_menu_accels;
  GtkWidget *magnifier;
  GtkWidget *vsurfplot;
  GtkWidget *lookuptable;
  GtkWidget *viewmip;
  GtkWidget *viewprofile;
  GtkWidget *viewlinesumx, *viewlinesumy;
  GtkWidget *viewzslab;
  GtkWidget *viewcontours;
  GtkWidget* viewstatop;
  GtkWidget* view_bright_contrast;
  
  	/* Menu: PROCESS */
  
  GtkWidget *process;
  GtkWidget *process_menu;
  GtkAccelGroup *process_menu_accels;
  GtkWidget *process1;
  GtkWidget *process2;
  GtkWidget *process3;
  GtkWidget *process3_menu;
  GtkAccelGroup *process3_menu_accels;
  GtkWidget *process3a;
  GtkWidget *process3b;
  GtkWidget *process3c;
  GtkWidget *process3d;
  GtkWidget *process3e;
  GtkWidget *process4;
  GtkWidget *process5;
  GtkWidget *process5_menu;
  GtkAccelGroup *process5_menu_accels;
  GtkWidget *process5a;
  GtkWidget *process5c;
  GtkWidget *process5e;
  GtkWidget *process5f;
  GtkWidget *processfft;
  GtkWidget *processfft_menu;
  GtkAccelGroup *processfft_menu_accels;
  GtkWidget *processffta;
  GtkWidget *processfftb;
  GtkWidget *processfftc;
  GtkWidget *processfftd;
  GtkWidget *processfilter;
  GtkWidget *processfilter_menu;
  GtkAccelGroup *processfilter_menu_accels;
  GtkWidget *processfilta;
  GtkWidget *processfiltb;
  GtkWidget *processfiltc;
  GtkWidget *processfiltd;
  GtkWidget *process6;
  GtkWidget *process7;
  GtkWidget *process8;
  
  	/* Menu SEGMENT */
	
  GtkWidget* segment1;
  GtkWidget* segment1_menu;
  GtkAccelGroup *segment1_menu_accels;
  GtkWidget* snake_segment;
  GtkWidget* snake_segment_menu;
  GtkAccelGroup *snake_menu_accels;
  GtkWidget *snake_0;
  GtkWidget *snake_1;
  GtkWidget *snake_2;
  GtkWidget* spider_segment;
  GtkWidget* hystthresh_segment;
  GtkWidget* watershed_segment;
  GtkWidget* regiongrow_segment;
  GtkWidget* growmerge_segment;
  GtkWidget* kmeans_segment;
    
  	/* Menu Measure */
	
  GtkWidget* measure;
  GtkWidget* measure_menu;
  GtkAccelGroup *measure_menu_accels;
  GtkWidget *m_statistics;
  GtkWidget *m_boundary;
  GtkWidget *m_clustcnt;
  GtkWidget *m_fractdim;
  GtkWidget *m_roseplot;
  GtkWidget *m_runlength;

	/* Menu: Special */

  GtkWidget* mspecial;
  GtkAccelGroup *mspecial_menu_accels;
  GtkWidget *m_vertebrae;
  GtkWidget *m_vessels;

	/* Menu: Macros */

  GtkWidget* mmacros;
  GtkAccelGroup *mmacros_menu_accels;
  GtkWidget *m_loadmacro;
  GtkWidget *m_deletemacro;
  GtkWidget *m_immdtmode;
    
  	/* Menu: HELP */
  
  GtkWidget *help1;
  GtkWidget *help1_menu;
  GtkAccelGroup *help1_menu_accels;
  GtkWidget *help_brows1;
  GtkWidget *about1;
  GtkWidget *imginfo1;
  GtkWidget* license1;
  GtkWidget* relnotes;
  GtkWidget *fixed1;
  GtkWidget *helppid1;
  
  /* begin main window */

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (window1), "window1", window1);
  gtk_window_set_title (GTK_WINDOW (window1), version_string);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window1), vbox1);



  /***************** BEGIN MENU ************************/

  menubar1 = gtk_menu_bar_new ();
  gtk_widget_ref (menubar1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "menubar1", menubar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menubar1);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar1, FALSE, FALSE, 5);

	/* FILE menu */

	file1 = Menuitem (window1,menubar1, NULL,"File","file1");
  	file1_menu = Makesubmenu (window1, file1, "file1_menu");
  	///file1_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (file1_menu));

	open1 = Menuitem (window1, file1_menu, on_open1_activate, "Open", "open1");
	open_multiple1 = Menuitem (window1, file1_menu, on_open_multiple1_activate, "Open Multiple", "open_multiple1");
	import1 = Menuitem (window1, file1_menu, on_import1_activate, "Import", "import1");
	rgbload1 = Menuitem (window1, file1_menu, on_rgbload1_activate, "3-Channel Load", "rgbload1");
	save1 = Menuitem (window1, file1_menu, on_save1_activate, "Save", "save1");
	close1 = Menuitem (window1, file1_menu, on_close1_activate, "Close", "close1");
	prefs1 = Menuitem (window1, file1_menu, on_prefs1_activate, "Settings", "prefs1");
	quit1 = Menuitem (window1, file1_menu, on_quit1_activate, "Quit", "quit1");

	/* EDIT menu */

  	editm = Menuitem (window1, menubar1, NULL, "Edit", "editm");
	edit_menu = Makesubmenu (window1, editm, "edit_menu");
  	///edit_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (edit_menu));

	edit_undo = Menuitem (window1, edit_menu, on_undo_activate, "UNDO", "edit_undo");
 	edit_trim = Menuitem (window1, edit_menu, on_trimimg_activate, "Trim Image", "edit_trim");
	edit_crop = Menuitem (window1, edit_menu, on_edit2_activate, "Crop", "edit2");
	edit_rotzoom = Menuitem (window1, edit_menu, on_edit3_activate, "Rotate/Zoom","edit3");
	edit_rebin = Menuitem (window1, edit_menu, on_rebin_activate, "Image Binning","edit7");
	edit_datatype = Menuitem (window1, edit_menu, on_edit4_activate, "Data type","edit4");
	edit_clickstk = Menuitem (window1, edit_menu, on_edit5_activate, "Click Stack","edit5");
	edit_values = Menuitem (window1, edit_menu, on_values_activate, "Image values","edit6");

	/* VIEW menu */

  	view1 = Menuitem (window1, menubar1, NULL, "View", "view1");
	view1_menu = Makesubmenu (window1, view1, "view1_menu");
  	///view1_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (view1_menu));

  	magnifier = Menuitem (window1, view1_menu, on_magnifier_activate, "Magnifier", "magnifier");
  	vsurfplot = Menuitem (window1, view1_menu, on_surfplot_activate, "3D surface plot", "vsurfplot");
// 	lookuptable = Menuitem (window1, view1_menu, on_lookup_activate, "Lookup Table", "lookuptable");
	lookuptable = Menuitem (window1, view1_menu, on_colormap_activate, "Colormaps", "colormaps");
	viewmip = Menuitem (window1, view1_menu, on_viewmip_activate, "MIP", "viewmip");
	viewprofile = Menuitem (window1, view1_menu, on_viewprofile_activate, "Profile", "viewprofile");
	viewlinesumx = Menuitem (window1, view1_menu, on_viewlinesumx_activate, "Line Avg (X)", "viewlinesumx");
	viewlinesumy = Menuitem (window1, view1_menu, on_viewlinesumy_activate, "Line Avg (Y)", "viewlinesumy");
	viewzslab = Menuitem (window1, view1_menu, on_viewzslab_activate, "Z Cut", "viewzslab");
	viewzslab = Menuitem (window1, view1_menu, on_viewcontours_activate, "Contour lines", "viewcontrs");
	viewstatop = Menuitem (window1, view1_menu, on_viewstatop_activate, "Histogram", "viewstatop");
// 	view_bright_contrast = Menuitem (window1, view1_menu, on_viewbc_activate, "Brightness/Contrast", "viewbc");
		 
	/* Process menu */
	
	process = Menuitem (window1, menubar1, NULL, "Process", "process");
	process_menu = Makesubmenu (window1, process, "process_menu");
  	///process_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (process_menu));

	process1 = Menuitem (window1, process_menu, on_invrt_activate, "Invert", "process_inv");
 	process2 = Menuitem (window1, process_menu, on_process2_activate, "Background removal", "process2");
 	process4 = Menuitem (window1, process_menu, on_process4_activate, "Cluster labeling", "process4");

 	process3 = Menuitem (window1, process_menu, NULL, "Threshold", "process3");
	process3_menu = Makesubmenu (window1, process3, "process3_menu");
  	///process3_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (process3_menu));
		process3a = Menuitem (window1, process3_menu, on_threshold1_activate, "Standard", "process3a");
		process3b = Menuitem (window1, process3_menu, on_threshold2_activate, "Bi-Level", "process3b");
		process3c = Menuitem (window1, process3_menu, on_threshold3_activate, "Auto:Iterative", "process3c");
		process3d = Menuitem (window1, process3_menu, on_threshold4_activate, "Auto:Otsu's Method", "process3d");
		process3e = Menuitem (window1, process3_menu, on_threshold5_activate, "Adaptive", "process3e");

 	process5 = Menuitem (window1, process_menu, NULL, "Binary", "process5");
	process5_menu = Makesubmenu (window1, process5, "process5_menu");
  	///process5_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (process5_menu));
		process5c = Menuitem (window1, process5_menu, on_morphop_activate, "Morphological Operators", "process5c");
		process5a = Menuitem (window1, process5_menu, on_skeletonize_activate, "Skeletonize", "process5a");
		process5e = Menuitem (window1, process5_menu, on_fillholes_activate, "Fill holes", "process5e");
		process5f = Menuitem (window1, process5_menu, on_centerfeat_activate, "Center features", "process5f");

 	processfft = Menuitem (window1, process_menu, NULL, "FFT", "processfft");
	processfft_menu = Makesubmenu (window1, processfft, "processfft_menu");
  	///processfft_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (processfft_menu));
		processffta = Menuitem (window1, processfft_menu, on_fft_activate, "FFT", "processffta");
		processfftb = Menuitem (window1, processfft_menu, on_ifft_activate, "Inverse FFT", "processfftb");
		processfftc = Menuitem (window1, processfft_menu, on_dht_activate, "DHT", "processfftc");
		processfftd = Menuitem (window1, processfft_menu, on_idht_activate, "Inverse DHT", "processfftd");

 	processfilter = Menuitem (window1, process_menu, NULL, "Filter", "processfilter");
	processfilter_menu = Makesubmenu (window1, processfilter, "processfilter_menu");
  	///processfilter_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (processfilter_menu));
		processfiltb = Menuitem (window1, processfilter_menu, on_convolve_activate, "Spatial-Domain Filters", "processfiltb");
		processfiltd = Menuitem (window1, processfilter_menu, on_afilter_activate, "Adaptive Spatial Filters", "processfiltd");
		processfiltc = Menuitem (window1, processfilter_menu, on_fourierfilt_activate, "Fourier Filters", "processfiltc");

 	process6 = Menuitem (window1, process_menu, on_process6_activate, "Equalize Histogram", "process6");
 	process7 = Menuitem (window1, process_menu, on_hough_activate, "Hough Transform", "process7");
 	process8 = Menuitem (window1, process_menu, on_imgmath_activate, "Image Math", "process8");

	/* Segment Menu */

	segment1 = Menuitem (window1, menubar1, NULL, "Segmentation", "segment");
	segment1_menu = Makesubmenu (window1, segment1, "segment_menu");
  	///segment1_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (segment1_menu));

 	snake_segment = Menuitem (window1, segment1_menu, NULL, "Snakes", "snake_segment0");
	snake_segment_menu = Makesubmenu (window1, snake_segment, "snake_menu");
  	///snake_menu_accels = gtk_menu_ensure_uline_roup (GTK_MENU (snake_segment_menu));
		snake_0 = Menuitem (window1, snake_segment_menu, on_snakeop_activate_0, "Greedy (Haidekker)", "snake0act");
		snake_1 = Menuitem (window1, snake_segment_menu, on_snakeop_activate_1, "Greedy (Williams/Shah)", "snake1act");
		snake_2 = Menuitem (window1, snake_segment_menu, on_snakeop_activate_2, "Force Balance", "snake2act");
	spider_segment = Menuitem (window1, segment1_menu, on_spiderop_activate, "Spider Op", "spider_segment");
 	hystthresh_segment = Menuitem (window1, segment1_menu, on_hystthresh_activate, "Hysteresis Threshold", "hystthresh_segment");
 	watershed_segment = Menuitem (window1, segment1_menu, on_watershed_activate, "Watershed", "watershed_segment");
 	regiongrow_segment = Menuitem (window1, segment1_menu, on_regiongrow_activate, "Region Growing", "regiongrow_segment");
 	growmerge_segment = Menuitem (window1, segment1_menu, on_growmerge_activate, "Grow-Merge", "growmerge_segment");
 	kmeans_segment = Menuitem (window1, segment1_menu, on_kmeans_activate, "k-Means Clustering", "kmeans_segment");

	/* MEASURE menu */
	
	measure = Menuitem (window1, menubar1, NULL, "Measure", "measure");
	measure_menu = Makesubmenu (window1, measure, "measure_menu");
  	///measure_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (measure_menu));

  	m_statistics = Menuitem (window1, measure_menu, on_stats_activate, "Statistics", "statistics");
  	m_boundary = Menuitem (window1, measure_menu, on_bound_activate, "Boundary length", "boundlen");
  	m_clustcnt = Menuitem (window1, measure_menu, on_clustcnt_activate, "Cluster maximum", "clustcnt");
  	m_fractdim = Menuitem (window1, measure_menu, on_fractdim_activate, "Fractal Dimesion", "fractdim");
	m_roseplot = Menuitem (window1, measure_menu, on_fft_rose_activate, "Rose Plot", "processffte");
	m_runlength = Menuitem (window1, measure_menu, on_runlength_activate, "Run Lengths", "processfilta");

	/* SPECIAL menu */
	
	mspecial = Menuitem (window1, menubar1, NULL, "Special", "mspecial");
	mspecial_menu = Makesubmenu (window1, mspecial, "mspecial_menu");
  	///mspecial_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (mspecial_menu));

	m_vertebrae = Menuitem (window1, mspecial_menu, on_vertebrae_activate, "CT vertebral bone", "m_vertebrae");
	m_vessels = Menuitem (window1, mspecial_menu, on_vesselanalysis_activate, "CT vascular grafts", "m_vessels");
	
	/* Macros Menu */

	mmacros = Menuitem (window1, menubar1, NULL, "Macros", "mmacros");
	mmacros_menu = Makesubmenu (window1, mmacros, "mmacros_menu");
  	///mmacros_menu_accels = gtk_menu_ensure_uline_group (GTK_MENU (mmacros_menu));

 	m_loadmacro = Menuitem (window1, mmacros_menu, on_loadmacro_activate, "Load Macro", "m_loadmacro");
 	m_deletemacro = Menuitem (window1, mmacros_menu, on_delmacro_activate, "Delete Macros", "m_delmacro");
 	m_immdtmode = Menuitem (window1, mmacros_menu, on_immdtmacro_activate, "Immediate execution", "m_immdtmacro");

	/* HELP menu */

  	help1 = Menuitem (window1, menubar1, NULL, "Help", "help1");
	help1_menu = Makesubmenu (window1, help1, "help1_menu");
  	///help1_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (help1_menu));

  	help_brows1 = Menuitem (window1, help1_menu, on_help1_activate, "CImage Help", "helpbrows");
  	about1 = Menuitem (window1, help1_menu, on_about1_activate, "About", "about");
  	imginfo1 = Menuitem (window1, help1_menu, on_imginfo1_activate, "Image Info", "imginfo1");
  	helppid1 = Menuitem (window1, help1_menu, on_pidinfo_activate, "Show PID", "pidinfo1");
	license1 = Menuitem (window1, help1_menu, on_license1_activate, "License", "license1");
	relnotes = Menuitem (window1, help1_menu, on_relnotes_activate, "Release Notes", "relnotes");

  /******************* END MENU *************************/


	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);
	
	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (hbox1), vbox2, FALSE, FALSE, 0);
	
	button1 = gtk_button_new_with_mnemonic ("Load");
	gtk_widget_show (button1);
	gtk_box_pack_start (GTK_BOX (vbox2), button1, FALSE, FALSE, 0);
	
	button2 = gtk_button_new_with_mnemonic ("Save");
	gtk_widget_show (button2);
	gtk_box_pack_start (GTK_BOX (vbox2), button2, FALSE, FALSE, 0);
	
	button3 = gtk_button_new_with_mnemonic ("Thumbnail");
	gtk_widget_show (button3);
	gtk_box_pack_start (GTK_BOX (vbox2), button3, FALSE, FALSE, 0);
	
	vseparator1 = gtk_vseparator_new ();
	gtk_widget_show (vseparator1);
	gtk_box_pack_start (GTK_BOX (hbox1), vseparator1, TRUE, TRUE, 0);
	
	vbox3 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox3);
	gtk_box_pack_start (GTK_BOX (hbox1), vbox3, TRUE, TRUE, 0);
	
	table1 = gtk_table_new (4, 2, FALSE);
	gtk_widget_show (table1);
	gtk_box_pack_start (GTK_BOX (vbox3), table1, TRUE, TRUE, 0);
	
	hsep1 = gtk_hseparator_new ();
	gtk_widget_show (hsep1);
	gtk_table_attach (GTK_TABLE (table1), hsep1, 0, 1, 2, 3,
						(GtkAttachOptions) (GTK_FILL),
						(GtkAttachOptions) (0), 0, 10);
	hsep2 = gtk_hseparator_new ();
	gtk_widget_show (hsep2);
	gtk_table_attach (GTK_TABLE (table1), hsep2, 1, 2, 2, 3,
						(GtkAttachOptions) (GTK_FILL),
						(GtkAttachOptions) (0), 0, 10);

	progressbar = gtk_progress_bar_new ();
	gtk_widget_show (progressbar);
	gtk_table_attach (GTK_TABLE (table1), progressbar, 1, 2, 3, 4,
						(GtkAttachOptions) (GTK_FILL),
						(GtkAttachOptions) (0), 0, 0);

	button4 = gtk_button_new_with_mnemonic ("Stop");
	gtk_widget_show (button4);
	gtk_table_attach (GTK_TABLE (table1), button4, 0, 1, 3, 4,
						(GtkAttachOptions) (0),
						(GtkAttachOptions) (0), 0, 0);

	label1 = gtk_label_new (" Z slice ");
	gtk_widget_show (label1);
	gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
						(GtkAttachOptions) (GTK_FILL),
						(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
	slice_spinbutton_adj = gtk_adjustment_new (0, 0, 0, 1, 10, 0);
	slice_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (slice_spinbutton_adj), 1, 0);
	gtk_widget_show (slice_spinbutton);
	gtk_table_attach (GTK_TABLE (table1), slice_spinbutton, 1, 2, 0, 1,
						(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
						(GtkAttachOptions) (0), 0, 0);

	label1 = gtk_label_new (" Zoom ");
	gtk_widget_show (label1);
	gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 1, 2,
						(GtkAttachOptions) (GTK_FILL),
						(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
	zoom_adj = gtk_adjustment_new (0, -3, 3, 1, 1, 0);
	zoom_hscale = gtk_hscale_new (GTK_ADJUSTMENT(zoom_adj));
	gtk_scale_set_digits (GTK_SCALE(zoom_hscale), 0);
	gtk_scale_set_value_pos (GTK_SCALE(zoom_hscale), GTK_POS_LEFT);
	gtk_widget_show (zoom_hscale);
	gtk_table_attach (GTK_TABLE (table1), zoom_hscale, 1, 2, 1, 2,
						(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
						(GtkAttachOptions) (0), 0, 0);

	/* To the right, create an (initially hidden) drawingarea for the thumbnail display */

	vseparator1 = gtk_vseparator_new ();
	gtk_widget_show (vseparator1);
	gtk_box_pack_start (GTK_BOX (hbox1), vseparator1, TRUE, TRUE, 0);
	vbox4 = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox1), vbox4, TRUE, TRUE, 0);

	thumbnail = gtk_drawing_area_new ();
	gtk_widget_ref (thumbnail);
	gtk_object_set_data_full (GTK_OBJECT (vbox4), "thumbnail", thumbnail,
								(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (thumbnail);
	gtk_container_add (GTK_CONTAINER (vbox4), thumbnail);
	gtk_widget_set_events (thumbnail, GDK_BUTTON_PRESS_MASK);

//	gtk_signal_connect (GTK_OBJECT (tn_window), "destroy",
//						GTK_SIGNAL_FUNC (destroy_thumbnail),NULL);
//	gtk_signal_connect (GTK_OBJECT (thumbnail), "button_press_event",
//						GTK_SIGNAL_FUNC (on_thumbnail_button_press_event),NULL);
						
	gtk_signal_connect (GTK_OBJECT (thumbnail), "configure_event",
						GTK_SIGNAL_FUNC (on_thumbnail_configure_event),NULL);
	gtk_signal_connect (GTK_OBJECT (thumbnail), "expose_event",
						GTK_SIGNAL_FUNC (on_thumbnail_expose_event),NULL);
	
	gtk_widget_set_size_request (vbox4, TN_SIZE+5, TN_SIZE+5);
	//gtk_window_set_resizable (GTK_WINDOW(vbox4), FALSE);		/* No user resizing allowed */
	gtk_widget_hide (vbox4);

	/* Below these elements, a status bar */

	statusbar1 = gtk_statusbar_new ();
	gtk_widget_show (statusbar1);
	gtk_box_pack_start (GTK_BOX (vbox1), statusbar1, FALSE, FALSE, 0);


	/* Signals that do not belong to the menu */

  	gtk_signal_connect (GTK_OBJECT (button1), "clicked", GTK_SIGNAL_FUNC (on_button1_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (button2), "clicked", GTK_SIGNAL_FUNC (on_button2_clicked), NULL);
  	gtk_signal_connect (GTK_OBJECT (button3), "clicked", GTK_SIGNAL_FUNC (on_button3_clicked), NULL);
 	gtk_signal_connect (GTK_OBJECT (button4), "clicked", GTK_SIGNAL_FUNC (on_stopbutton_clicked), NULL);
   	gtk_signal_connect (GTK_OBJECT (slice_spinbutton), "changed",
                      GTK_SIGNAL_FUNC (on_slice_spinbutton_changed), slice_spinbutton);
   	gtk_signal_connect (GTK_OBJECT (zoom_hscale), "value-changed",
                      GTK_SIGNAL_FUNC (on_zoom_changed), zoom_adj);
	gtk_signal_connect (GTK_OBJECT (window1), "destroy",
                      GTK_SIGNAL_FUNC (on_quit1_activate), NULL);

	/* Before we are done, publish the drawingarea */

	tn_img.drawable = thumbnail;
	tn_img.parent = vbox4;
	tn_img.pixbuf = NULL;
	tn_img.data = NULL;

  return window1;
}




GtkWidget* open_imagewindow (image_type *img, int mwx, int mwy)
{
GtkWidget *imgwnd;
GtkWidget *vbox;
GtkWidget *scrolledwindow1;
GtkWidget *viewport1;
GtkWidget *drawingarea1;
GtkRuler *hruler, *vruler;
GtkWidget *table;


	imgwnd = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (imgwnd), "Crystal Image Display");

	/* Request default window size. If width & height are set,
		and in reasonable range, say, less than 640, use those values.
		Otherwise, choose some default size */

	if ((img->width>0) && (img->width<=640) && (img->height>0) && (img->height<=640))
		gtk_window_set_default_size (GTK_WINDOW(imgwnd), img->width+28, img->height+28);
	else
		gtk_window_set_default_size (GTK_WINDOW(imgwnd), 400,300);
	
	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_container_add (GTK_CONTAINER (imgwnd), scrolledwindow1);
	
	viewport1 = gtk_viewport_new (NULL, NULL);
	gtk_widget_show (viewport1);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), viewport1);

	table = gtk_table_new (2, 2, FALSE); 
	gtk_widget_ref (table);
	gtk_object_set_data_full (GTK_OBJECT (imgwnd), "dspytbl", table, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (table);
	gtk_container_add (GTK_CONTAINER (viewport1), table);

	drawingarea1 = gtk_drawing_area_new ();
	gtk_widget_show (drawingarea1);
	//gtk_container_add (GTK_CONTAINER (vbox), drawingarea1);
	//gtk_box_pack_start (GTK_BOX (table), drawingarea1, FALSE, TRUE, 0);
	gtk_table_attach (GTK_TABLE (table), drawingarea1, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 
					4, 4);						/* H- and V- padding */
	gtk_widget_set_size_request (drawingarea1, img->width, img->height);

	/* This is not handled by Glade: Under GTK+, low-level events are
	   not sent to the application by default. We must specifically enable the 
		mouse events. Even more, as of GTK-2.0, the "can-focus" property is needed
		to handle keypress events. However, this ash the downside of emitting an expose
		event for the entire are whenever the window receives the focus. Very inefficient.
		Yuck. */
	
	g_object_set (drawingarea1, "can-focus", 1, NULL);			/* New in GTK-2.0: Needed for keypress events */
	gtk_widget_set_events (drawingarea1, GDK_EXPOSURE_MASK
										| GDK_ENTER_NOTIFY_MASK
										| GDK_LEAVE_NOTIFY_MASK
										| GDK_BUTTON_PRESS_MASK
										| GDK_BUTTON_RELEASE_MASK
										| GDK_POINTER_MOTION_MASK
										| GDK_POINTER_MOTION_HINT_MASK
										| GDK_KEY_PRESS_MASK
										| GDK_SCROLL_MASK
										/*| GDK_ALL_EVENTS_MASK */);


/* TODO: The rulers still need to be developed */

/*	hruler = gtk_hruler_new ();
	gtk_table_attach (GTK_TABLE (table), GTK_WIDGET(hruler), 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 
					4, 4);						/* H- and V- padding */
/*	gtk_widget_show (GTK_WIDGET(hruler));
	gtk_ruler_set_metric (hruler, GTK_PIXELS);*/
//  	gtk_ruler_set_range (hruler, 0, img->width, 0, img->width);*/
// 	vruler = gtk_vruler_new ();
// 	gtk_table_attach (GTK_TABLE (table), GTK_WIDGET(vruler), 1, 2, 0, 1,
//                     (GtkAttachOptions) (0),
//                     (GtkAttachOptions) (0), 
// 					4, 4);						/* H- and V- padding */
// 	gtk_widget_show (GTK_WIDGET(vruler));
// 	gtk_ruler_set_metric (vruler, GTK_PIXELS);
//  	gtk_ruler_set_range (vruler, 0, img->height, 0, img->height);



	dp (2, "FYI. New image window %08X, drawingarea %08X\n", imgwnd, drawingarea1);

	/* A note about these callbacks. Since these are window-specific, we pass the pointer
		to the associated image as user_data. This allows to access the GUI through 
		img->parent.
	*/

	gtk_signal_connect (GTK_OBJECT (imgwnd), "destroy",
						GTK_SIGNAL_FUNC (signal_close_file), img);
	
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "expose_event",
						GTK_SIGNAL_FUNC (on_drawingarea1_expose_event), img);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "configure_event",
						GTK_SIGNAL_FUNC (on_drawingarea1_configure_event), img);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "button_press_event",
						GTK_SIGNAL_FUNC (on_drawingarea1_clicked_event), img);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "button_release_event",
						GTK_SIGNAL_FUNC (on_drawingarea1_click_released_event), img);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "motion_notify_event",
						GTK_SIGNAL_FUNC (on_drawingarea1_motion_notify_event), img);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "key_press_event",
						GTK_SIGNAL_FUNC (on_drawingarea1_key_press_event), img);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "enter_notify_event",
						GTK_SIGNAL_FUNC (on_drawingarea1_enter_notify_event), img);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "leave_notify_event",
						GTK_SIGNAL_FUNC (on_drawingarea1_leave_notify_event), img);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "scroll_event",
						GTK_SIGNAL_FUNC (on_drawingarea1_scroll_event), img);


	/* Set the GUI components of the associated image */

	img->parent = imgwnd;				/* The toplevel window of the display */
	img->drawable = drawingarea1;		/* The drawable (actually, the draweable is drawingarea1->window */


  return imgwnd;
}

